package org.processmining.dataawarecnetminer.extension.conformance;

import org.processmining.dataawarecnetminer.common.MinerContext;
import org.processmining.dataawarecnetminer.exception.MiningException;
import org.processmining.dataawarecnetminer.model.FrequencyAwareCausalNet;
import org.processmining.dataawarecnetminer.model.FrequencyAwareDataCausalNet;

/**
 * Computation of the conformance information for the C-Net. For example,
 * frequencies of activities and input-/output bindings.
 * 
 * @author F. Mannhardt
 *
 */
public interface ConformanceHeuristic {

	/**
	 * Computes the frequencies of input- and output bindings for the C-Net.
	 * 
	 * @param cNet
	 * @throws MiningException
	 */
	FrequencyAwareCausalNet computeFrequencies(MinerContext context, FrequencyAwareCausalNet cNet) throws MiningException;

	/**
	 * Computes the frequencies of input- and output bindings for the DC-Net.
	 * 
	 * @param dcNet
	 * @throws MiningException
	 */
	FrequencyAwareDataCausalNet computeFrequencies(MinerContext context, FrequencyAwareDataCausalNet dcNet) throws MiningException;

}