package org.processmining.dataawarecnetminer.extension.decision;

import java.util.Map;
import java.util.Set;

import org.processmining.dataawarecnetminer.extension.DefaultHeuristicPlugin;
import org.processmining.dataawarecnetminer.extension.HeuristicPlugin;
import org.processmining.dataawarecnetminer.interactive.methods.MethodConfiguration;
import org.processmining.dataawarecnetminer.model.DataRelationStorage;
import org.processmining.dataawarecnetminer.model.EventRelationStorage;
import org.processmining.datadiscovery.DecisionTreeConfig;
import org.processmining.datadiscovery.OverlappingLocalDecisionTreesImpl;
import org.processmining.datadiscovery.ProjectedLog;
import org.processmining.datadiscovery.RuleDiscovery;
import org.processmining.datadiscovery.RuleDiscovery.Rule;
import org.processmining.datadiscovery.estimators.Type;
import org.processmining.framework.plugin.annotations.KeepInProMCache;

@KeepInProMCache
@HeuristicPlugin
@DefaultHeuristicPlugin
public class DecisionHeuristicExtensionOverlapping extends DefaultDecisionHeuristicFactory {

	public DecisionHeuristic newInstance(EventRelationStorage eventStorage, DataRelationStorage dataStorage) {
		return new QualityAwareDecisionHeuristicImpl(eventStorage, dataStorage) {

			protected RuleDiscovery<? extends Rule> newRuleDiscovery(ProjectedLog projectedLog,
					DecisionTreeConfig discoveryConfig, Map<String, Type> attributesTypes,
					Map<String, Set<String>> literalValues, int estimateSize) {
				return new OverlappingLocalDecisionTreesImpl(discoveryConfig, projectedLog, attributesTypes,
						literalValues, estimateSize);
			}

			protected DecisionHeuristicConfig configureDecisionMiner(MethodConfiguration methodConfig) {
				DecisionHeuristicConfig config = super.configureDecisionMiner(methodConfig);
				config.setUsesDecisionThreshold(false); // does not support quality assessment 
				return config;
			}

		};
	}

	public String getName() {
		return "Overlapping Decision Rules (No Score)";
	}

	public String getReference() {
		return "Overlapping decision rule discovery as described in: F Mannhardt et al. Decision Mining Revisited - Discovering Overlapping Rules. CAiSE 2016. DOI: 10.1007/978-3-319-39696-5_23";
	}

}