package org.processmining.dataawarecnetminer.extension.decision;

import java.util.Map;
import java.util.Set;

import org.processmining.dataawarecnetminer.common.MinerContext;
import org.processmining.dataawarecnetminer.exception.MiningException;
import org.processmining.dataawarecnetminer.interactive.methods.MethodConfiguration;
import org.processmining.dataawarecnetminer.mining.decision.DataAwareCausalNetMiner;
import org.processmining.dataawarecnetminer.model.DataRelationStorage;
import org.processmining.dataawarecnetminer.model.EventRelationStorage;
import org.processmining.dataawarecnetminer.model.FrequencyAwareCausalNet;
import org.processmining.dataawarecnetminer.model.FrequencyAwareDataCausalNet;
import org.processmining.datadiscovery.DecisionTreeConfig;
import org.processmining.datadiscovery.ProjectedLog;
import org.processmining.datadiscovery.RuleDiscovery;
import org.processmining.datadiscovery.RuleDiscovery.Rule;
import org.processmining.datadiscovery.RuleDiscoveryException;
import org.processmining.datadiscovery.estimators.Type;

abstract class QualityAwareDecisionHeuristicImpl extends DefaultDecisionHeuristic {

	private final DataAwareCausalNetMiner dataAwareCausalMiner;

	public QualityAwareDecisionHeuristicImpl(EventRelationStorage eventStorage, DataRelationStorage dataStorage) {
		super();
		dataAwareCausalMiner = new DataAwareCausalNetMiner(eventStorage, dataStorage) {

			protected RuleDiscovery<? extends Rule> createRuleDiscovery(ProjectedLog projectedLog,
					DecisionTreeConfig discoveryConfig, Map<String, Type> attributesTypes,
					Map<String, Set<String>> literalValues, int estimateSize) {
				return newRuleDiscovery(projectedLog, discoveryConfig, attributesTypes, literalValues, estimateSize);
			}

		};
	}

	public FrequencyAwareDataCausalNet computeDecisions(MinerContext context, MethodConfiguration configuration,
			FrequencyAwareCausalNet causalNet) throws MiningException {
		dataAwareCausalMiner.setConfig(configureDecisionMiner(configuration));
		try {
			return dataAwareCausalMiner.mineDataAwareCausalNet(causalNet, context.getExecutor());
		} catch (RuleDiscoveryException e) {
			throw new MiningException(e);
		}
	}

	protected DecisionHeuristicConfig configureDecisionMiner(MethodConfiguration methodConfig) {
		DecisionHeuristicConfig config = new DecisionHeuristicConfig();
		config.setDecisionThreshold(methodConfig.getDecisionThreshold());
		config.setWriteThreshold(methodConfig.getDecisionThreshold()); // TODO does this need its own slider?
		config.setDataDiscoveryConfig(methodConfig.getDataDiscoveryConfig());
		config.setDataDiscoveryAttributes(methodConfig.getDataDiscoveryAttributes());
		return config;
	}

	abstract protected RuleDiscovery<? extends Rule> newRuleDiscovery(ProjectedLog projectedLog,
			DecisionTreeConfig discoveryConfig, Map<String, Type> attributesTypes,
			Map<String, Set<String>> literalValues, int estimateSize);

}