package org.processmining.dataawarecnetminer.extension.dependencies;

import org.processmining.dataawarecnetminer.common.MinerContext;
import org.processmining.dataawarecnetminer.extension.DefaultHeuristicPlugin;
import org.processmining.dataawarecnetminer.extension.HeuristicPlugin;
import org.processmining.dataawarecnetminer.interactive.methods.MethodConfiguration;
import org.processmining.dataawarecnetminer.mining.classic.HeuristicsCausalGraphBuilder;
import org.processmining.dataawarecnetminer.model.DependencyAwareCausalGraph;
import org.processmining.dataawarecnetminer.model.EventRelationStorage;
import org.processmining.framework.plugin.annotations.KeepInProMCache;

@KeepInProMCache
@HeuristicPlugin
@DefaultHeuristicPlugin
public class DependencyHeuristicExtensionFHMImpl extends DefaultDependencyHeuristicFactory {

	private static class DependencyHeuristicFHMImpl extends DefaultDependencyHeuristic {

		private HeuristicsCausalGraphBuilder fhmGraphBuilder;

		public DependencyHeuristicFHMImpl(EventRelationStorage eventStorage) {
			super();
			this.fhmGraphBuilder = new HeuristicsCausalGraphBuilder(eventStorage);
		}

		public DependencyAwareCausalGraph computeDependencies(MinerContext context, MethodConfiguration configuration) {
			fhmGraphBuilder.setConfig(configureHeuristicCausalGraphMiner(configuration));
			return fhmGraphBuilder.build(context);
		}

		protected DependencyHeuristicConfig configureHeuristicCausalGraphMiner(MethodConfiguration configuration) {
			DependencyHeuristicConfig config = new DependencyHeuristicConfig();
			config.setDependencyThreshold(configuration.getDependencyThreshold());
			config.setL1Threshold(configuration.getL1Threshold());
			config.setL2Threshold(configuration.getL2Threshold());
			config.setRelativeToBestThreshold(configuration.getRelativeToBestThreshold());

			config.setObservationThreshold(configuration.getFrequencyThreshold());
			config.setAllTasksConnected(configuration.isAllTasksConnected());
			config.setAcceptedTasksConnected(configuration.isAcceptedTasksConnected());
			return config;
		}

	}

	public DependencyHeuristic newInstance(EventRelationStorage eventStorage) {
		return new DependencyHeuristicFHMImpl(eventStorage);
	}

	public String getName() {
		return "Flexible Heuristics Miner";
	}

	public String getReference() {
		return "AJMM Weijters, JTS Ribeiro. Flexible Heuristic Miner. DOI: 10.1109/CIDM.2011.5949453";
	}

}