package org.processmining.dataawarecnetminer.interactive;

import java.awt.BasicStroke;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;

import javax.swing.AbstractAction;
import javax.swing.AbstractButton;
import javax.swing.Action;
import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;
import javax.swing.UIManager;
import javax.swing.plaf.basic.BasicButtonUI;
import javax.swing.plaf.basic.BasicTabbedPaneUI;

class ClosableTabbedPane extends JTabbedPane {

	private static final long serialVersionUID = 8652127862715121886L;
	
	private final class RemoveAction extends AbstractAction {

		private static final long serialVersionUID = 1L;

		private final Component component;

		private RemoveAction(Component panel) {
			this.component = panel;
		}

		public void actionPerformed(ActionEvent e) {
			int index = indexOfComponent(component);				
			if (index != -1) {
				setSelectedIndex(index-1);
				removeTabAt(index);
			}
		}
	}

	private final static class TitleWithCloseButton extends JPanel {

		private final static class CloseButton extends JButton {

			private static final long serialVersionUID = 1L;

			private final MouseListener buttonMouseListener = new MouseAdapter() {
				public void mouseEntered(MouseEvent e) {
					Component component = e.getComponent();
					if (component instanceof AbstractButton) {
						AbstractButton button = (AbstractButton) component;
						button.setBorderPainted(true);
					}
				}

				public void mouseExited(MouseEvent e) {
					Component component = e.getComponent();
					if (component instanceof AbstractButton) {
						AbstractButton button = (AbstractButton) component;
						button.setBorderPainted(false);
					}
				}
			};

			public CloseButton(final JTabbedPane tabbedPane) {
				int size = 17;
				setPreferredSize(new Dimension(size, size));
				setToolTipText("Close");
				setUI(new BasicButtonUI());
				setContentAreaFilled(false);
				setFocusable(false);
				setBorder(BorderFactory.createEtchedBorder());
				setBorderPainted(false);
				addMouseListener(buttonMouseListener);
				setRolloverEnabled(true);
			}

			public void updateUI() {
			}

			protected void paintComponent(Graphics g) {
				super.paintComponent(g);
				Graphics2D g2 = (Graphics2D) g.create();
				if (getModel().isPressed()) {
					g2.translate(1, 1);
				}
				g2.setStroke(new BasicStroke(2));
				g2.setColor(Color.BLACK);
				if (getModel().isRollover()) {
					g2.setColor(Color.RED);
				}
				int delta = 6;
				g2.drawLine(delta, delta, getWidth() - delta - 1, getHeight() - delta - 1);
				g2.drawLine(getWidth() - delta - 1, delta, delta, getHeight() - delta - 1);
				g2.dispose();
			}
		}

		private static final long serialVersionUID = 1L;

		public TitleWithCloseButton(final JTabbedPane tabbedPane, final String title, final Action closeAction) {
			setOpaque(false);
			JLabel label = new JLabel(title);
			label.setBorder(BorderFactory.createEmptyBorder(0, 0, 0, 5));
			add(label, BorderLayout.CENTER);
			JButton closeButton = new CloseButton(tabbedPane);
			closeButton.addActionListener(new ActionListener() {

				public void actionPerformed(ActionEvent e) {
					closeAction.actionPerformed(e);
				}
			});
			add(closeButton, BorderLayout.EAST);
		}

	}

	private final static class TitleWithoutCloseButton extends JPanel {

		private static final long serialVersionUID = 1L;

		public TitleWithoutCloseButton(final JTabbedPane tabbedPane, final String title) {
			setOpaque(false);
			JLabel label = new JLabel(title);
			label.setBorder(BorderFactory.createEmptyBorder(0, 0, 0, 5));
			add(label, BorderLayout.CENTER);
		}

	}

	public ClosableTabbedPane() {
		this(TOP, WRAP_TAB_LAYOUT);
	}
	
	public ClosableTabbedPane(int tabPlacement) {
		 this(tabPlacement, WRAP_TAB_LAYOUT);
	}

	public ClosableTabbedPane(int tabPlacement, int tabLayoutPolicy) {
		super(tabPlacement, tabLayoutPolicy);		
		setOpaque(false);
		setUI(new BasicTabbedPaneUI() {

			protected void installDefaults() {
				Color defaultColor = UIManager.getColor("TabbedPane.selected");
				UIManager.put("TabbedPane.selected", Color.WHITE);
				super.installDefaults();
				lightHighlight = Color.BLACK;
				highlight = Color.BLACK;
				shadow = Color.WHITE;
				darkShadow = Color.BLACK;
				focus = Color.WHITE;

				tabInsets = new Insets(0, 4, 0, 4);
				selectedTabPadInsets = new Insets(2, 2, 1, 1);
				tabAreaInsets = new Insets(1, 2, 0, 2);
				contentBorderInsets = new Insets(1, 2, 2, 3);
				UIManager.put("TabbedPane.selected", defaultColor);
			}

		});
	}


	public void addClosableTab(String title, Component component, String tip) {
		addTab(title, null, component, tip);
		int index = indexOfComponent(component);
		setTabComponentAt(index, new TitleWithCloseButton(this, title, new RemoveAction(component)));
		setBackgroundAt(index, Color.WHITE);
	}

	public void addNonClosableTabe(String title, Component component, String tip) {
		addTab(title, null, component, tip);
		int index = indexOfComponent(component);
		setTabComponentAt(index, new TitleWithoutCloseButton(this, title));
		setBackgroundAt(index, Color.WHITE);
	}
	
}