package org.processmining.dataawarecnetminer.interactive.methods.cnet;

import javax.swing.JComponent;

import org.processmining.dataawarecnetminer.common.MinerContext;
import org.processmining.dataawarecnetminer.exception.MiningException;
import org.processmining.dataawarecnetminer.mining.classic.HeuristicsCausalNetMiner;
import org.processmining.dataawarecnetminer.model.DataAwareCausalGraph;
import org.processmining.dataawarecnetminer.model.DataRelationStorage;
import org.processmining.dataawarecnetminer.model.DependencyAwareCausalGraph;
import org.processmining.dataawarecnetminer.model.EventRelationStorage;
import org.processmining.dataawarecnetminer.model.FrequencyAwareCausalNet;
import org.processmining.dataawarecnetminer.model.FrequencyAwareDataCausalNet;
import org.processmining.dataawarecnetminer.visualization.DataScoreEdgeDecorator;
import org.processmining.dataawarecnetminer.visualization.GuardExpressionNodeDecorator;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.models.dcnet.DataAwareCausalNet;
import org.processmining.plugins.dcnet.DataAwareCausalNetVisualization;
import org.processmining.plugins.utils.ProvidedObjectHelper;

public class DataAwareDCNetMethod extends DataDependencyGraphMethod {

	private FrequencyAwareDataCausalNet dcNet;

	public DataAwareDCNetMethod() {
		super();
		getConfiguration().setUsesLongTerm(true);
		getConfiguration().setUsesNoTask(true);
		getConfiguration().setUsesAcceptedTasks(true);
		getConfiguration().setUsesBindings(true);
		getConfiguration().setUsesConformance(true);
		getConfiguration().setUsesDecision(true);
	}

	@Override
	public JComponent mine(MinerContext context) throws MiningException {
		context.getProgress().setCaption("discover dependencies ...");
		DataAwareCausalGraph dependencyGraph = buildDataDependencyGraph(context);

		context.getProgress().setCaption("determine bindings & guards ...");
		FrequencyAwareDataCausalNet initialDCNet = mineDataCausalNetWithGuards(context, getEventStorage(),
				getDataStorage(), dependencyGraph);

		context.getProgress().setCaption("compute conformance information ...");
		dcNet = getConformanceHeuristic().computeFrequencies(context, initialDCNet);

		DataAwareCausalNetVisualization netVisualization = new DataAwareCausalNetVisualization(dcNet.getDCNet(),
				new GuardExpressionNodeDecorator(dcNet),
				new DataScoreEdgeDecorator(dcNet, dependencyGraph, getEventStorage()));

		return netVisualization.getVisualization();
	}

	@Override
	public void save(PluginContext context, String fileName) throws MiningException {
		DataAwareCausalNet causalNet = dcNet.getDCNet();
		ProvidedObjectHelper.publish(context, fileName, causalNet, DataAwareCausalNet.class, false);
		ProvidedObjectHelper.setFavorite(context, causalNet);
	}

	protected FrequencyAwareDataCausalNet mineDataCausalNetWithGuards(MinerContext context,
			final EventRelationStorage eventStorage, DataRelationStorage dataStorage,
			DependencyAwareCausalGraph dependencyGraph) throws MiningException {
		FrequencyAwareCausalNet causalNet = mineCausalNet(context, eventStorage, dependencyGraph);
		return getDecisionHeuristic().computeDecisions(context, getConfiguration(), causalNet);
	}

	protected FrequencyAwareCausalNet mineCausalNet(MinerContext context, final EventRelationStorage eventStorage,
			DependencyAwareCausalGraph dependencyGraph) throws MiningException {
		HeuristicsCausalNetMiner causalNetMiner = new HeuristicsCausalNetMiner(eventStorage);
		causalNetMiner.getConfig().setBindingsThreshold(getConfiguration().getBindingsThreshold());
		causalNetMiner.getConfig().setConsiderLongDistanceRelations(getConfiguration().isLongTerm());
		causalNetMiner.getConfig().setLongTermDependencyThreshold(getConfiguration().getDependencyThreshold());
		causalNetMiner.getConfig().setNoTasksUnconnected(getConfiguration().isNoTasksUnconnected());
		FrequencyAwareCausalNet causalNet = causalNetMiner.mineCausalNet(context, dependencyGraph,
				getBindingsHeuristic());
		return causalNet;
	}

	public String toString() {
		return "Data Causal net (DC-net)";
	}

}