package org.processmining.dataawarecnetminer.interactive.methods.cnet;

import javax.swing.JComponent;

import org.processmining.dataawarecnetminer.common.MinerContext;
import org.processmining.dataawarecnetminer.converter.CausalNetToPetrinet;
import org.processmining.dataawarecnetminer.exception.MiningException;
import org.processmining.dataawarecnetminer.interactive.util.DotHelper;
import org.processmining.dataawarecnetminer.model.DataRelationStorage;
import org.processmining.dataawarecnetminer.model.DependencyAwareCausalGraph;
import org.processmining.dataawarecnetminer.model.EventRelationStorage;
import org.processmining.dataawarecnetminer.model.FrequencyAwareCausalNet;
import org.processmining.dataawarecnetminer.model.FrequencyAwareDataCausalNet;
import org.processmining.datapetrinets.DataPetriNet;
import org.processmining.datapetrinets.DataPetriNetsWithMarkings;
import org.processmining.datapetrinets.visualization.graphviz.DPNGraphvizConverter;
import org.processmining.datapetrinets.visualization.graphviz.DPNGraphvizConverter.DPNAsDot;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.models.connections.petrinets.behavioral.FinalMarkingConnection;
import org.processmining.models.connections.petrinets.behavioral.InitialMarkingConnection;
import org.processmining.models.semantics.petrinet.Marking;
import org.processmining.plugins.utils.ProvidedObjectHelper;

public class DataAwareDPNMinerMethod extends DataAwareDCNetMethod {

	private DataPetriNetsWithMarkings dpn;

	public DataAwareDPNMinerMethod() {
		super();
		getConfiguration().setUsesConformance(false);
	}

	@Override
	public JComponent mine(MinerContext context) throws MiningException {
		dpn = mineDPN(context, getEventStorage(), getDataStorage());
		DPNAsDot dpnAsDot = DPNGraphvizConverter.convertDPN(dpn, dpn.getInitialMarking(), dpn.getFinalMarkings());
		return new DotHelper().createPanel(dpnAsDot.getDot());
	}

	@Override
	public void save(PluginContext context, String fileName) throws MiningException {
		ProvidedObjectHelper.publish(context, fileName, dpn, DataPetriNet.class, false);
		ProvidedObjectHelper.publish(context, "Initial Marking", dpn.getInitialMarking(), Marking.class, false);
		ProvidedObjectHelper.publish(context, "Initial Marking Connection",
				new InitialMarkingConnection(dpn, dpn.getInitialMarking()), InitialMarkingConnection.class, false);
		ProvidedObjectHelper.publish(context, "Final Marking", dpn.getFinalMarkings()[0], Marking.class, false);
		ProvidedObjectHelper.publish(context, "Final Marking Connection",
				new FinalMarkingConnection(dpn, dpn.getFinalMarkings()[0]), FinalMarkingConnection.class, false);
		ProvidedObjectHelper.setFavorite(context, dpn);
	}

	protected DataPetriNetsWithMarkings mineDPN(MinerContext context, EventRelationStorage eventStorage,
			DataRelationStorage dataStorage) throws MiningException {
		DependencyAwareCausalGraph dependencyGraph = buildDataDependencyGraph(context);

		FrequencyAwareCausalNet causalNet = mineCausalNet(context, eventStorage, dependencyGraph);
		FrequencyAwareDataCausalNet dataCausalNet = getDecisionHeuristic().computeDecisions(context,
				getConfiguration(), causalNet);

		return new CausalNetToPetrinet().convert(dataCausalNet.getDCNet());
	}

	public String toString() {
		return "Data Petri net (DPN)";
	}

}