package org.processmining.dataawarecnetminer.interactive.methods.cnet;

import javax.swing.JComponent;

import org.processmining.dataawarecnetminer.common.MinerContext;
import org.processmining.dataawarecnetminer.converter.CausalNetToPetrinet;
import org.processmining.dataawarecnetminer.exception.MiningException;
import org.processmining.dataawarecnetminer.interactive.util.DotHelper;
import org.processmining.dataawarecnetminer.model.DataAwareCausalGraph;
import org.processmining.dataawarecnetminer.model.FrequencyAwareCausalNet;
import org.processmining.datapetrinets.DataPetriNet;
import org.processmining.datapetrinets.DataPetriNet.PetrinetWithMarkings;
import org.processmining.datapetrinets.visualization.graphviz.DPNGraphvizConverter;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.models.connections.petrinets.behavioral.FinalMarkingConnection;
import org.processmining.models.connections.petrinets.behavioral.InitialMarkingConnection;
import org.processmining.models.graphbased.directed.petrinet.Petrinet;
import org.processmining.models.semantics.petrinet.Marking;
import org.processmining.plugins.graphviz.dot.Dot;
import org.processmining.plugins.utils.ProvidedObjectHelper;

public class DataAwarePetrinetMinerMethod extends DataAwareCNetMethod {

	private PetrinetWithMarkings net;

	public DataAwarePetrinetMinerMethod() {
		super();
		getConfiguration().setUsesConformance(false);
	}

	public JComponent mine(MinerContext context) throws MiningException {
		net = minePetrinet(context);
		Dot dot = DPNGraphvizConverter.convertDPN(DataPetriNet.Factory.viewAsDataPetriNet(net.getNet()),
				net.getInitialMarking(), net.getFinalMarkings()).getDot();
		return new DotHelper().createPanel(dot);
	}

	private PetrinetWithMarkings minePetrinet(MinerContext context) throws MiningException {
		DataAwareCausalGraph dependencyGraph = buildDataDependencyGraph(context);
		FrequencyAwareCausalNet dataAwareCNet = mineCNet(context, getEventStorage(), dependencyGraph);
		return new CausalNetToPetrinet().convert(dataAwareCNet.getCNet());
	}

	@Override
	public void save(PluginContext context, String fileName) {
		ProvidedObjectHelper.publish(context, fileName, net.getNet(), Petrinet.class, false);
		ProvidedObjectHelper.publish(context, "Initial Marking", net.getInitialMarking(), Marking.class, false);
		ProvidedObjectHelper.publish(context, "Initial Marking Connection",
				new InitialMarkingConnection(net.getNet(), net.getInitialMarking()), InitialMarkingConnection.class,
				false);
		ProvidedObjectHelper.publish(context, "Final Marking", net.getFinalMarkings()[0], Marking.class, false);
		ProvidedObjectHelper.publish(context, "Final Marking Connection",
				new FinalMarkingConnection(net.getNet(), net.getFinalMarkings()[0]), FinalMarkingConnection.class,
				false);
		ProvidedObjectHelper.setFavorite(context, net.getNet());
	}

	public String toString() {
		return "Petri net";
	}

}