package org.processmining.dataawarecnetminer.interactive.methods.cnet;

import java.text.MessageFormat;
import java.util.HashMap;
import java.util.Map;

import javax.swing.JComponent;

import org.deckfour.xes.classification.XEventClass;
import org.processmining.dataawarecnetminer.common.MinerContext;
import org.processmining.dataawarecnetminer.exception.MiningException;
import org.processmining.dataawarecnetminer.interactive.methods.AbstractMiningMethod;
import org.processmining.dataawarecnetminer.interactive.util.DotHelper;
import org.processmining.dataawarecnetminer.model.DependencyAwareCausalGraph;
import org.processmining.dataawarecnetminer.model.DependencyRelation;
import org.processmining.dataawarecnetminer.model.EventRelationStorage;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.models.causalgraph.Relation;
import org.processmining.models.causalgraph.SimpleCausalGraph;
import org.processmining.plugins.graphviz.dot.Dot;
import org.processmining.plugins.graphviz.dot.DotEdge;
import org.processmining.plugins.graphviz.dot.DotNode;
import org.processmining.plugins.utils.ProvidedObjectHelper;

public class DependencyGraphMethod extends AbstractMiningMethod {

	private DependencyAwareCausalGraph dependencyGraph;

	public DependencyGraphMethod() {
		super();
		getConfiguration().setUsesDependency(true);
		getConfiguration().setUsesL1Threshold(true);
		getConfiguration().setUsesL2Threshold(true);
		getConfiguration().setUsesRelativeToBestThreshold(true);
		getConfiguration().setUsesFrequency(true);
		getConfiguration().setUsesAllTask(true);
		getConfiguration().setUsesAcceptedTasks(true);
	}

	@Override
	public JComponent mine(MinerContext context) throws MiningException {
		DotHelper dotHelper = new DotHelper();
		return dotHelper.createPanel(buildDot(dotHelper, getEventStorage(),
				getDependencyHeuristic().computeDependencies(context, getConfiguration())));
	}

	@Override
	public void save(PluginContext context, String fileName) throws MiningException {
		ProvidedObjectHelper.publish(context, fileName, dependencyGraph, SimpleCausalGraph.class, false);
		ProvidedObjectHelper.setFavorite(context, dependencyGraph);
	}

	private Dot buildDot(DotHelper helper, EventRelationStorage eventRelations,
			DependencyAwareCausalGraph dependencyGraph) {

		Dot dot = new Dot();
		Map<XEventClass, DotNode> activityToNode = new HashMap<XEventClass, DotNode>();

		for (Relation relation : dependencyGraph.getCausalRelations()) {
			DotEdge dependency = helper.addDependency(eventRelations, dot, activityToNode, relation);
			DependencyRelation dependencyRelation = dependencyGraph.getDependencyRelation(relation);
			double dependencyValue = dependencyRelation.getDependencyValue();
			dependency.setLabel(MessageFormat.format("{0,number,#.###}", dependencyValue));
		}

		return dot;
	}

	public String toString() {
		return "Dependency graph";
	}

}