package org.processmining.dataawarecnetminer.interactive.methods.relations;

import java.awt.Color;
import java.text.NumberFormat;
import java.util.HashMap;
import java.util.Locale;
import java.util.Map;

import javax.swing.JComponent;

import org.deckfour.xes.classification.XEventClass;
import org.processmining.dataawarecnetminer.common.MinerContext;
import org.processmining.dataawarecnetminer.exception.MiningException;
import org.processmining.dataawarecnetminer.interactive.methods.AbstractMiningMethod;
import org.processmining.dataawarecnetminer.interactive.util.DotHelper;
import org.processmining.dataawarecnetminer.model.EventRelationStorage;
import org.processmining.dataawarecnetminer.visualization.ColorScheme;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.models.causalgraph.Relation;
import org.processmining.plugins.graphviz.dot.Dot;
import org.processmining.plugins.graphviz.dot.DotEdge;
import org.processmining.plugins.graphviz.dot.DotNode;
import org.processmining.plugins.utils.ProvidedObjectHelper;

import com.google.common.collect.ImmutableMultiset;
import com.google.common.collect.Multiset.Entry;
import com.google.common.collect.Multisets;

public final class DirectlyFollowsMethod extends AbstractMiningMethod {

	public DirectlyFollowsMethod() {
		super();
		getConfiguration().setUsesFrequency(true);
	}

	public JComponent mine(MinerContext context) throws MiningException {
		DotHelper dotHelper = new DotHelper();
		return dotHelper.createPanel(doMine(dotHelper, getEventStorage()));
	}

	public void save(PluginContext context, String fileName) throws MiningException {
		Dot dot = doMine(new DotHelper(), getEventStorage());
		ProvidedObjectHelper.publish(context, fileName, dot, Dot.class, false);
		ProvidedObjectHelper.setFavorite(context, dot);
	}

	private Dot doMine(DotHelper dotHelper, EventRelationStorage eventRelations) {
		Dot dot = new Dot();

		Map<XEventClass, DotNode> activityToNode = new HashMap<XEventClass, DotNode>();

		int maxNodeCount = Multisets.copyHighestCountFirst(eventRelations.getEventClassOccurence()).entrySet()
				.iterator().next().getCount();

		double maxCount = -1;
		ImmutableMultiset<Relation> sortedRelations = Multisets
				.copyHighestCountFirst(eventRelations.getDirectlyFollowsRelations());
		for (Entry<Relation> entry : sortedRelations.entrySet()) {
			maxCount = Math.max(maxCount, entry.getCount());
			if (entry.getCount() >= eventRelations.countTraces() * getConfiguration().getFrequencyThreshold()) {
				Relation relation = entry.getElement();
				DotNode sourceNode = dotHelper.addNodeIfNeeded(eventRelations, dot, relation.getSource(),
						activityToNode);
				colorNode(eventRelations, relation.getSource(), sourceNode, maxNodeCount);
				DotNode targetNode = dotHelper.addNodeIfNeeded(eventRelations, dot, relation.getTarget(),
						activityToNode);
				colorNode(eventRelations, relation.getTarget(), targetNode, maxNodeCount);

				DotEdge edge = dot.addEdge(sourceNode, targetNode);
				edge.setOption("penwidth", String.format(Locale.ENGLISH, "%.2f", dotHelper.calcEdgeWidth(entry.getCount(), maxCount)));
				Color color = ColorScheme.BLUE_SINGLE_HUE.getColorFromGradient(entry.getCount() / maxCount);
				edge.setOption("fillcolor", convertColor(color));
				edge.setOption("color", convertColor(color));
				edge.setLabel(NumberFormat.getIntegerInstance().format(entry.getCount()));
			}
		}

		return dot;
	}

	private void colorNode(EventRelationStorage eventRelations, XEventClass nodeClass, DotNode node, int maxCount) {
		int count = eventRelations.countOccurence(nodeClass);
		Color color = ColorScheme.BLUE_SINGLE_HUE.getColorFromGradient(count / (double) maxCount);
		node.setOption("fillcolor", convertColor(color));
		node.setOption("style", "filled");
		node.setOption("fontcolor", convertColor(determineFontColor(null, color)));
	}

	private static final Color determineFontColor(final Color textColor, final Color bgColor) {
		if (textColor == null) {
			double val = Math.sqrt(.299 * Math.pow(bgColor.getRed(), 2) + .587 * Math.pow(bgColor.getGreen(), 2)
					+ .114 * Math.pow(bgColor.getBlue(), 2));
			return (val < 130) ? Color.WHITE : Color.BLACK;
		} else {
			return textColor;
		}
	}

	private static String convertColor(Color color) {
		return String.format(Locale.ENGLISH, "#%02X%02X%02X", color.getRed(), color.getGreen(), color.getBlue());
	}

	public String toString() {
		return "Directly-follows Graph";
	}

}