package org.processmining.dataawarecnetminer.interactive.methods.relations;

import java.text.MessageFormat;
import java.util.HashMap;
import java.util.Map;

import javax.swing.JComponent;

import org.deckfour.xes.classification.XEventClass;
import org.processmining.dataawarecnetminer.common.MinerContext;
import org.processmining.dataawarecnetminer.exception.MiningException;
import org.processmining.dataawarecnetminer.interactive.methods.AbstractMiningMethod;
import org.processmining.dataawarecnetminer.interactive.util.DotHelper;
import org.processmining.dataawarecnetminer.model.EventRelationStorage;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.models.causalgraph.Relation;
import org.processmining.plugins.graphviz.dot.Dot;
import org.processmining.plugins.graphviz.dot.DotEdge;
import org.processmining.plugins.graphviz.dot.DotNode;
import org.processmining.plugins.utils.ProvidedObjectHelper;

import com.google.common.collect.ImmutableMultiset;
import com.google.common.collect.Multiset.Entry;
import com.google.common.collect.Multisets;

public class LongDistanceDependencyMethod extends AbstractMiningMethod {

	public LongDistanceDependencyMethod() {
		super();
		getConfiguration().setUsesFrequency(true);
		getConfiguration().setUsesDependency(true);
	}

	public JComponent mine(MinerContext context)
			throws MiningException {
		DotHelper dotHelper = new DotHelper();
		return dotHelper.createPanel(doMine(dotHelper, getEventStorage()));
	}

	public void save(PluginContext context,
			String fileName) throws MiningException {
		Dot dot = doMine(new DotHelper(), getEventStorage());
		ProvidedObjectHelper.publish(context, fileName, dot, Dot.class, false);
		ProvidedObjectHelper.setFavorite(context, dot);
	}

	protected Dot doMine(DotHelper helper, EventRelationStorage eventRelations) {
		Dot dot = new Dot();

		Map<XEventClass, DotNode> activityToNode = new HashMap<XEventClass, DotNode>();

		double maxCount = -1;
		ImmutableMultiset<Relation> sortedRelations = Multisets
				.copyHighestCountFirst(eventRelations.getDirectlyFollowsRelations());
		for (Entry<Relation> entry : sortedRelations.entrySet()) {
			maxCount = Math.max(maxCount, entry.getCount());
			if (entry.getCount() >= eventRelations.countTraces() * getConfiguration().getFrequencyThreshold()) {
				Relation relation = entry.getElement();
				if (!eventRelations.includesStartOrEnd(relation)) {
					double dependencyValue = eventRelations.getLongDistanceDependency(relation).getDependencyValue();
					if (dependencyValue > getConfiguration().getDependencyThreshold()) {
						DotNode sourceNode = helper.addNodeIfNeeded(eventRelations, dot, relation.getSource(),
								activityToNode);
						DotNode targetNode = helper.addNodeIfNeeded(eventRelations, dot, relation.getTarget(),
								activityToNode);
						DotEdge edge = dot.addEdge(sourceNode, targetNode);
						edge.setLabel(MessageFormat.format("{0,number,#.###}", dependencyValue));
					}
				}
			}
		}

		return dot;
	}

	public String toString() {
		return "Long Distance Relations";
	}

}