package org.processmining.dataawarecnetminer.model;

import org.processmining.datapetrinets.expression.GuardExpression;
import org.processmining.models.causalgraph.Relation;

import weka.classifiers.AbstractClassifier;
import weka.classifiers.Evaluation;

public class DataRelationImpl implements DataRelation {

	private final Relation relation;
	private final GuardExpression expression;

	private final double qualityMean;
	private final double qualityVariance;

	private final AbstractClassifier classifier;
	private final Evaluation evaluation;

	public DataRelationImpl(GuardExpression expression, Relation relation, double qualityMean, double qualityVariance,
			AbstractClassifier classifier, Evaluation evaluation) {
		this.expression = expression;
		this.relation = relation;
		this.qualityMean = qualityMean;
		this.qualityVariance = qualityVariance;
		this.classifier = classifier;
		this.evaluation = evaluation;
	}

	public DataRelationImpl(GuardExpression trueInstance, Relation relation) {
		this.expression = trueInstance;
		this.relation = relation;
		this.qualityMean = 0;
		this.qualityVariance = 0;
		this.classifier = null;
		this.evaluation = null;
	}

	public AbstractClassifier getClassifier() {
		if (classifier != null) {
			return classifier;
		} else {
			return null;
		}
	}

	public Evaluation getEvaluation() {
		if (evaluation != null) {
			return evaluation;
		} else {
			return null;
		}
	}

	public GuardExpression getCondition() {
		return expression;
	}

	public Relation getRelation() {
		return relation;
	}

	public boolean isConditionTrue() {
		return GuardExpression.Factory.trueInstance().equals(getCondition());
	}

	public double getQualityScore() {
		return qualityMean;
	}

	public double getQualityVariance() {
		return qualityVariance;
	}

	public String getQualityScoreName() {
		return "Cohen's Kappa";
	}

	public String getEvaluationDetails() {
		try {
			return getEvaluation().toMatrixString() + "\n" + getEvaluation().toSummaryString(false) + "\n"
					+ getEvaluation().toClassDetailsString();
		} catch (Exception e) {
			return e.getMessage();
		}
	}

}