package org.processmining.dataawarecnetminer.plugins;

import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.dataawarecnetminer.converter.CausalNetToPetrinet;
import org.processmining.datapetrinets.DataPetriNet.PetrinetWithMarkings;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginLevel;
import org.processmining.models.cnet.CausalNet;
import org.processmining.models.connections.petrinets.behavioral.FinalMarkingConnection;
import org.processmining.models.connections.petrinets.behavioral.InitialMarkingConnection;
import org.processmining.models.graphbased.directed.petrinet.Petrinet;
import org.processmining.models.semantics.petrinet.Marking;
import org.processmining.plugins.utils.ProvidedObjectHelper;

/**
 * Conversion from {@link CausalNet} to {@link Petrinet}
 * 
 * @author F. Mannhardt
 *
 */
public final class CNetToPetriNetPlugin {

	@Plugin(name = "Convert Causal net (C-Net) to Petri net", parameterLabels = { "Causal net" }, //
			level = PluginLevel.PeerReviewed, returnLabels = { "Petri net" }, returnTypes = {
					Petrinet.class }, userAccessible = true, mostSignificantResult = 1, help = "Converts a CausalNet to a Petri net")
	@UITopiaVariant(affiliation = UITopiaVariant.EHV, author = " F. Mannhardt", email = "f.mannhardt@tue.nl")
	public Petrinet convertCNetToPN(PluginContext context, CausalNet causalNet) {
		PetrinetWithMarkings netWithMarkings = convertCNetToPN(causalNet);
		String label = ProvidedObjectHelper.getProvidedObjectLabel(context, causalNet);
		Petrinet net = netWithMarkings.getNet();
		InitialMarkingConnection initialMarkingConnection = new InitialMarkingConnection(net,
				netWithMarkings.getInitialMarking());
		context.addConnection(initialMarkingConnection);
		ProvidedObjectHelper.publish(context, "Initial marking of " + label, netWithMarkings.getInitialMarking(),
				Marking.class, false);
		FinalMarkingConnection finalMarkingConnection = new FinalMarkingConnection(net,
				netWithMarkings.getFinalMarkings()[0]);
		context.addConnection(finalMarkingConnection);
		ProvidedObjectHelper.publish(context, "Final marking of " + label, netWithMarkings.getFinalMarkings()[0],
				Marking.class, false);
		context.getFutureResult(0).setLabel("PN converted from " + label);
		return net;
	}

	public static PetrinetWithMarkings convertCNetToPN(CausalNet causalNet) {
		return new CausalNetToPetrinet().convert(causalNet);
	}

}
