package org.processmining.dataawarecnetminer.util;

import java.util.ArrayDeque;
import java.util.Deque;
import java.util.HashSet;
import java.util.Set;

import org.deckfour.xes.classification.XEventClass;
import org.processmining.models.causalgraph.Relation;

public final class GraphUtils {

	private GraphUtils() {
	}

	public static boolean isConnected(Set<Relation> graph, Relation relation, XEventClass startEventClass,
			XEventClass endEventClass) {
		return hasPathTo(graph, startEventClass, relation.getSource())
				&& hasPathTo(graph, relation.getTarget(), endEventClass);
	}

	public static boolean hasPathTo(Set<Relation> graph, XEventClass source, XEventClass target) {
		if (source.equals(target)) {
			return true;
		}
		Set<XEventClass> visitedNodes = new HashSet<>();
		Deque<XEventClass> stack = new ArrayDeque<>();
		stack.push(source);
		while (!stack.isEmpty()) {
			XEventClass current = stack.pop();

			if (current.equals(target)) { // found node continue with others paths
				return true;
			}

			visitedNodes.add(current);

			for (Relation relation : graph) {
				if (relation.getSource().equals(current)) {
					if (!visitedNodes.contains(relation.getTarget())) {
						stack.push(relation.getTarget());
					}
				}
			}
		}
		return false;
	}

}
