package org.processmining.dataawarecnetminer.visualization;

import java.awt.Color;
import java.awt.Dialog.ModalityType;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.text.MessageFormat;
import java.util.prefs.Preferences;

import javax.swing.AbstractAction;
import javax.swing.JComponent;
import javax.swing.JFileChooser;
import javax.swing.JPopupMenu;
import javax.swing.JTextArea;
import javax.swing.KeyStroke;
import javax.swing.filechooser.FileNameExtensionFilter;

import org.apache.commons.lang3.StringUtils;
import org.deckfour.xes.classification.XEventClass;
import org.processmining.dataawarecnetminer.model.DataAwareCausalGraph;
import org.processmining.dataawarecnetminer.model.DataRelation;
import org.processmining.dataawarecnetminer.model.EventRelationStorage;
import org.processmining.dataawarecnetminer.model.FrequencyAwareCausalNet;
import org.processmining.dataawarecnetminer.model.FrequencyAwareDataCausalNet;
import org.processmining.datadiscovery.visualizers.PrefuseTreeVisualization;
import org.processmining.datadiscovery.visualizers.PrefuseTreeVisualization.TreePanel;
import org.processmining.framework.util.ui.widgets.helper.ProMUIHelper;
import org.processmining.models.causalgraph.Relation;
import org.processmining.models.cnet.CNetNode;
import org.processmining.plugins.dcnet.EdgeStyle;

import weka.classifiers.evaluation.ThresholdCurve;
import weka.core.Drawable;
import weka.core.Instances;
import weka.core.Utils;
import weka.gui.visualize.PlotData2D;
import weka.gui.visualize.ThresholdVisualizePanel;

public final class DataScoreEdgeDecorator extends DependencyScoreEdgeDecorator {

	private static final class ShowROCAction extends AbstractAction {
		private static final long serialVersionUID = 1L;

		private final DataRelation dataRelation;
		private final JPopupMenu menu;

		private ShowROCAction(String name, DataRelation dataRelation, JPopupMenu menu) {
			super(name);
			this.dataRelation = dataRelation;
			this.menu = menu;
		}

		public void actionPerformed(ActionEvent e) {
			// generate curve
			ThresholdCurve tc = new ThresholdCurve();
			Instances result = tc.getCurve(dataRelation.getEvaluation().predictions(), 1);

			try {
				// plot curve
				ThresholdVisualizePanel vmc = new ThresholdVisualizePanel();
				vmc.setShowAttBars(false);
				vmc.setROCString("(Area under ROC = " + Utils.doubleToString(tc.getROCArea(result), 2) + ")");
				vmc.setName(result.relationName());
				PlotData2D tempd = new PlotData2D(result);
				tempd.setPlotName(result.relationName());
				tempd.addInstanceNumberAttribute();
				// specify which points are connected
				boolean[] cp = new boolean[result.numInstances()];
				for (int n = 1; n < cp.length; n++)
					cp[n] = true;
				tempd.setConnectPoints(cp);
				// add plot
				vmc.addPlot(tempd);
				vmc.setPreferredSize(new Dimension(600, 600));

				UIUtil.showCustom(menu, vmc, "ROC for Relation " + dataRelation.getRelation().toString(),
						ModalityType.MODELESS);

			} catch (Exception e1) {
				ProMUIHelper.showErrorMessage(menu, e1.getMessage(), "Could not draw ROC", e1);
			}
		}

	}

	private static final class ShowPRCAction extends AbstractAction {
		private static final long serialVersionUID = 1L;

		private final DataRelation dataRelation;
		private final JPopupMenu menu;

		private ShowPRCAction(String name, DataRelation dataRelation, JPopupMenu menu) {
			super(name);
			this.dataRelation = dataRelation;
			this.menu = menu;
		}

		public void actionPerformed(ActionEvent e) {
			// generate curve
			ThresholdCurve tc = new ThresholdCurve();
			Instances result = tc.getCurve(dataRelation.getEvaluation().predictions(), 1);

			try {
				// plot curve
				ThresholdVisualizePanel vmc = new ThresholdVisualizePanel();
				vmc.setShowAttBars(false);
				vmc.setROCString("(Area under PRC = " + Utils.doubleToString(tc.getPRCArea(result), 2) + ")");
				vmc.setName(result.relationName());
				PlotData2D tempd = new PlotData2D(result);
				tempd.setPlotName(result.relationName());
				tempd.addInstanceNumberAttribute();
				// specify which points are connected
				boolean[] cp = new boolean[result.numInstances()];
				for (int n = 1; n < cp.length; n++)
					cp[n] = true;
				tempd.setConnectPoints(cp);
				// add plot
				vmc.addPlot(tempd);
				vmc.setXIndex(8);
				vmc.setYIndex(7);
				vmc.setPreferredSize(new Dimension(600, 600));

				UIUtil.showCustom(menu, vmc, "PRC for Relation " + dataRelation.getRelation().toString(),
						ModalityType.MODELESS);

			} catch (Exception e1) {
				ProMUIHelper.showErrorMessage(menu, e1.getMessage(), "Could not draw ROC", e1);
			}
		}

	}

	private static final class ShowClassifierEvaluationAction extends AbstractAction {

		private static final long serialVersionUID = 1L;

		private final DataRelation dataRelation;
		private final JPopupMenu menu;

		private ShowClassifierEvaluationAction(String name, DataRelation dataRelation, JPopupMenu menu) {
			super(name);
			this.dataRelation = dataRelation;
			this.menu = menu;
		}

		public void actionPerformed(ActionEvent e) {
			JTextArea evaluationPanel = new JTextArea(dataRelation.getEvaluationDetails());
			evaluationPanel.setFont(new Font("monospaced", Font.PLAIN, 12));
			UIUtil.showCustom(menu, evaluationPanel,
					"Classifier Evaluation for Relation " + dataRelation.getRelation().toString(),
					ModalityType.MODELESS);
		}
	}

	private static final class ShowClassifierAction extends AbstractAction {

		private static final long serialVersionUID = 1L;

		private final DataRelation dataRelation;
		private JPopupMenu menu;

		private ShowClassifierAction(String name, DataRelation dataRelation, JPopupMenu menu) {
			super(name);
			this.dataRelation = dataRelation;
			this.menu = menu;
		}

		public void actionPerformed(ActionEvent e) {
			if (dataRelation.getClassifier() instanceof Drawable) {
				try {
					final TreePanel treePanel = createTreePanel(dataRelation.getRelation().toString(),
							(Drawable) dataRelation.getClassifier());
					treePanel.setToolTipText("Use CTRL+I to save as image");
					UIUtil.showCustom(menu, treePanel,
							"Classifier for Relation " + dataRelation.getRelation().toString(), ModalityType.MODELESS);
				} catch (Exception e1) {
					ProMUIHelper.showErrorMessage(menu, e1.getMessage(), "Could not draw classifier", e1);
				}
			}
		}

		private TreePanel createTreePanel(final String label, final Drawable drawable) throws Exception {
			PrefuseTreeVisualization treeVis = new PrefuseTreeVisualization();
			final TreePanel treePanel = treeVis.display(drawable.graph(), label);
			treePanel.getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW)
					.put(KeyStroke.getKeyStroke(KeyEvent.VK_I, InputEvent.CTRL_MASK), "saveImage");
			treePanel.getActionMap().put("saveImage", new AbstractAction() {

				private static final long serialVersionUID = 1L;

				public void actionPerformed(ActionEvent e) {
					JFileChooser fileChooser = new JFileChooser();
					Preferences preferences = Preferences.userRoot().node("org.processmining.graphviz");
					fileChooser.setCurrentDirectory(
							new File(preferences.get("lastUsedFolder", new File(".").getAbsolutePath())));
					FileNameExtensionFilter pngFilter = new FileNameExtensionFilter("PNG Image", "png");
					fileChooser.addChoosableFileFilter(pngFilter);
					fileChooser.setFileFilter(pngFilter);

					int option = fileChooser.showSaveDialog(treePanel);
					if (option == JFileChooser.APPROVE_OPTION) {
						String filename = fileChooser.getSelectedFile().getPath();
						if (!filename.endsWith(".png")) {
							filename = filename + ".png";
						}
						try (FileOutputStream output = new FileOutputStream(filename)) {
							treePanel.saveImage(output, "png", 1.0d);
						} catch (IOException e1) {
							ProMUIHelper.showErrorMessage(menu, e1.getMessage(), "Could not save classifier", e1);
						}
					}
				}

			});
			return treePanel;
		}
	}

	private final DataAwareCausalGraph dependencyGraph;
	private final EventRelationStorage eventStorage;

	public DataScoreEdgeDecorator(FrequencyAwareCausalNet frequencyAwareCausalNet, DataAwareCausalGraph dependencyGraph,
			EventRelationStorage eventStorage) {
		super(frequencyAwareCausalNet.getCNet(), dependencyGraph, eventStorage);
		this.dependencyGraph = dependencyGraph;
		this.eventStorage = eventStorage;
	}

	public DataScoreEdgeDecorator(FrequencyAwareDataCausalNet frequencyAwareCausalNet,
			DataAwareCausalGraph dependencyGraph, EventRelationStorage eventStorage) {
		super(frequencyAwareCausalNet.getDCNet(), dependencyGraph, eventStorage);
		this.dependencyGraph = dependencyGraph;
		this.eventStorage = eventStorage;
	}

	public Color getEdgeColor(CNetNode source, CNetNode target) {
		DataRelation dataRelation = getDataRelation(source, target);
		if (dataRelation != null) {
			return Color.RED;
		} else {
			return super.getEdgeColor(source, target);
		}
	}

	public double getEdgeWidth(CNetNode source, CNetNode target) {
		return super.getEdgeWidth(source, target);
	}

	public EdgeStyle getEdgeStyle(CNetNode source, CNetNode target) {
		return super.getEdgeStyle(source, target);
	}

	public String getEdgeLabel(CNetNode source, CNetNode target) {
		DataRelation dataRelation = getDataRelation(source, target);
		if (dataRelation != null) {
			return MessageFormat.format("{0} ({1,number,#.##} \u00B1 {2,number,#.###})",
					StringUtils.abbreviate(dataRelation.getCondition().toPrettyString(1), 30),
					getPerformanceMeasure(dataRelation), getPerformanceVariance(dataRelation));
		} else {
			return super.getEdgeLabel(source, target);
		}
	}

	private double getPerformanceVariance(DataRelation dataRelation) {
		return dataRelation.getQualityVariance();
	};

	private double getPerformanceMeasure(DataRelation dataRelation) {
		return dataRelation.getQualityScore();
	}

	public JPopupMenu getEdgeMenu(CNetNode source, CNetNode target) {

		final JPopupMenu menu = super.getEdgeMenu(source, target);

		final DataRelation dataRelation = getDataRelation(source, target);

		if (dataRelation != null) {
			menu.add(new ShowClassifierAction("Show Classifier", dataRelation, menu));
			menu.add(new ShowClassifierEvaluationAction("Show Evaluation", dataRelation, menu));
			menu.add(new ShowROCAction("Show ROC", dataRelation, menu));
			menu.add(new ShowPRCAction("Show PRC", dataRelation, menu));
		}

		return menu;
	}

	private DataRelation getDataRelation(CNetNode source, CNetNode target) {
		XEventClass sourceClass = eventStorage.getEventClassesById().get(source.getLabel());
		XEventClass targetClass = eventStorage.getEventClassesById().get(target.getLabel());
		Relation relation = Relation.Factory.create(sourceClass, targetClass);
		DataRelation dataRelation = dependencyGraph.getDataRelation(relation);
		return dataRelation;
	}

}