package org.processmining.dataawarecnetminer.visualization;

import java.text.MessageFormat;

import org.deckfour.xes.classification.XEventClass;
import org.processmining.dataawarecnetminer.model.DependencyAwareCausalGraph;
import org.processmining.dataawarecnetminer.model.DependencyRelation;
import org.processmining.dataawarecnetminer.model.EventRelationStorage;
import org.processmining.models.causalgraph.Relation;
import org.processmining.models.cnet.CNetNode;
import org.processmining.models.cnet.CausalNet;
import org.processmining.models.dcnet.DataAwareCausalNet;
import org.processmining.plugins.dcnet.DefaultEdgeDecorator;

public class DependencyScoreEdgeDecorator extends DefaultEdgeDecorator {

	private final DependencyAwareCausalGraph causalGraph;
	private final EventRelationStorage relations;

	public DependencyScoreEdgeDecorator(CausalNet causalNet, DependencyAwareCausalGraph causalGraph,
			EventRelationStorage relations) {
		super(causalNet);
		this.causalGraph = causalGraph;
		this.relations = relations;
	}

	public DependencyScoreEdgeDecorator(DataAwareCausalNet dataCausalNet, DependencyAwareCausalGraph causalGraph,
			EventRelationStorage relations) {
		super(dataCausalNet);
		this.causalGraph = causalGraph;
		this.relations = relations;
	}

	public String getEdgeLabel(CNetNode source, CNetNode target) {
		DependencyRelation dependencyRelation = getRelation(source, target);
		if (dependencyRelation != null) {
			return MessageFormat.format("{0,number,#.##}", dependencyRelation.getDependencyValue());
		} else {
			// might be a long term dependency
			return "long term dependency";
		}
	}

	private DependencyRelation getRelation(CNetNode source, CNetNode target) {
		XEventClass sourceClass = relations.getEventClassesById().get(source.getLabel());
		XEventClass targetClass = relations.getEventClassesById().get(target.getLabel());
		return causalGraph.getDependencyRelation(Relation.Factory.create(sourceClass, targetClass));
	}

}