package org.processmining.dataawarecnetminer.visualization;

public enum TransformationMode {

	LINEAR("Linear", new TransformationFunction() {

		public double transform(double maxValue, double value) {
			if (maxValue == 0.0d) {
				return 0.0;
			}
			return value / maxValue;
		}

	}), SQRT("Square root", new TransformationFunction() {

		public double transform(double maxValue, double value) {
			if (maxValue == 0.0d) {
				return 0.0;
			}
			 return Math.sqrt(value) / Math.sqrt(maxValue);
		}

	}), LOG("Logarithm", new TransformationFunction() {

		public double transform(double maxValue, double value) {
			if (maxValue == 0.0d) {
				return 0.0;
			}
			return Math.log(value) / Math.log(maxValue);
		}

	});

	private final String desc;
	private final TransformationFunction transformation;

	private TransformationMode(String desc, TransformationFunction transformatio) {
		this.desc = desc;
		this.transformation = transformatio;
	}

	@Override
	public String toString() {
		return desc;
	}

	public double transform(double maxValue, double value) {
		return transformation.transform(maxValue, value);
	}

}